<?php
/*================================================*/
/* Набор функций для разбора документов HTML.     */
/* Автор: Игорь Сергеевич Орещенков, 2017-2018 г. */
/* Среда разработки: PHP 5.6.23 + AkelPad 4.9.7   */
/*================================================*/
require_once 'iaparser.php';
/*=====================================================================*/
/* Получение значения HTML-элемента, находящегося между тегами.        */
/* Вызов: &$s - ссылка на обрабатываемую строку,                       */
/*        &$tagb - ссылка на массив с информацией об открывающем теге, */
/*        &$tage - ссылка на массив с информацией о закрывающем теге.  */
/* Возврат: подстрока, находящаяся между открывающим и закрывающим     */
/*          тегами или FALSE в случае ошибки.                          */
/*=====================================================================*/
function iaFoundElementContent (&$s, &$tagb, &$tage)
{
	$pb = iaFoundTagAfterPos ($tagb);
	$pe = iaFoundTagPos ($tage);
	return ($pb !== FALSE and $pe !== FALSE)? substr ($s, $pb, $pe - $pb): FALSE;
}
/*=================================================================*/
/* Получение значения атрибута тега.                               */
/* Вызов: &$tag - ссылка на массив с информацией о найденном теге, */
/*        $attr - имя атрибута тега.                               */
/* Возврат: значение атрибута или FALSE, если атрибут              */
/*          с именем $attr отсутствует.                            */
/*=================================================================*/
function iaFoundTagAttr (&$tag, $attr)
{
	return (is_array ($tag) and is_array ($tag['attr']) and array_key_exists ($attr, $tag['attr']))? $tag['attr'][$attr]: FALSE;
}
/*=======================================================*/
/* Получение номера позиции начала найденного тега.      */
/* Вызов: &$tag - массив с информацией о найденном теге. */
/* Возврат: номер позиции начала найденного тега.        */
/*=======================================================*/
function iaFoundTagPos (&$tag) {
	return (
		is_array ($tag) and
		array_key_exists ('pos', $tag) and $tag['pos'] !== FALSE
		)? $tag['pos']: FALSE;
}
/*=========================================================*/
/* Получение номера позиции, следующей за найденным тегом. */
/* Вызов: &$tag - массив с информацией о найденном теге.   */
/* Возврат: номер позиции, следующей за найденным тегом.   */
/*=========================================================*/
function iaFoundTagAfterPos (&$tag) {
	return (
		is_array ($tag) and
		array_key_exists ('pos', $tag) and $tag['pos'] !== FALSE and
		array_key_exists ('len', $tag) and $tag['len'] !== FALSE
		)? $tag['pos'] + $tag['len']: FALSE;
}
/*=========================================================================*/
/* Поиск одного из указанных HTML-тегов.                                   */
/* Вызов: $p    - номер позиции в строке, с которой начинать поиск,        */
/*        &$s   - ссылка на строку, в которой должен осуществляться поиск, */
/*        $tags - массив искомых тегов.                                    */
/* Возврат: массив с информацией о найденном теге.                         */
/*=========================================================================*/
function iaFindTagOneOf ($p, &$s, $tags) {
	if ($p !== FALSE):
		if (is_array ($p)):
			$p = iaFoundTagAfterPos ($p);
		endif;
		$tag = iaFindTagAny ($p, $s);
		while ($tag !== FALSE and !(is_array ($tags)? in_array ($tag['name'], $tags): $tag['name'] == $tags)):
			if ($tag['len'] > 0):
				$p = $tag['pos'] + $tag['len'];
				$tag = iaFindTagAny ($p, $s);
			else:
				$tag = FALSE;
			endif;
		endwhile;
	else:
		$tag = FALSE;
	endif;
	return $tag;
}
/*=======================================================================*/
/* Поиск любого HTML-тега.                                               */
/* Вызов: $p  - номер позиции в строке, с которой начинать поиск,        */
/*        &$s - ссылка на строку, в которой должен осуществляться поиск. */
/* Возврат: массив с информацией о найденном теге.                       */
/*=======================================================================*/
function iaFindTagAny ($p, &$s) {
	$spc = " \t\n\r";	// пробельные символы
	$tag = FALSE;		// информация о теге
	/* Поиск начала тега. */
	$l = strlen ($s);
	do {
		$p = strpos ($s, '<', $p);
		$c = ($p !== FALSE and $p < $l - 1)? $s{++$p}: ' ';
		$f = (('A' <= $c and $c <= 'Z') or ('a' <= $c and $c <= 'z') or $c == '/');
	} while ($p !== FALSE and !$f);
	/* Получение названия тега. */
	if ($p !== FALSE and $f):
		$tag = array (
			'pos' => FALSE, 	// позиция начала тега
			'len' => FALSE, 	// длина тега
			'name' => FALSE,	// имя тега
			'attr' => FALSE		// массив атрибутов
		);
		$tag['pos'] = $p - 1;
		/* Поиск конца названия тега. */
		$r = $p;
		$p = iaSkipTo ($s, ">$spc", $p + 1, $l);
		if ($p !== FALSE):
			$tag['name'] = substr ($s, $r, $p - $r);
		endif;
		/* Поиск атрибутов тега. */
		$p = iaSkipOver ($s, $spc, $p, $l);
		while ($p !== FALSE and $s{$p} != '>'):
			$anp = $p;	/* позиция начала названия атрибута */
			$anl = 0;	/* длина названия атрибута */
			$avp = -1;	/* позиция начала значения атрибута */
			$avl = 0;	/* длина значения атрибута */
			/* Поиск конца названия атрибута. */
			$p = iaSkipTo ($s, "=>$spc", $p, $l);
			if ($p !== FALSE):
				$anl = $p - $anp;
			endif;
			/* Пропуск пробелов после конца названия атрибута. */
			$p = iaSkipOver ($s, $spc, $p, $l);
			if ($p !== FALSE and $s{$p} == '='):
				/* Поиск начала значения атрибута. */
				$p = iaSkipOver ($s, $spc, $p + 1, $l);
				if ($p !== FALSE):
					$q = $s{$p};
					if ($q == '\'' or $q == '"'):
						/* Значение атрибута заключено в кавычки. */
						$p++;	// пропуск открывающей кавычки
						if ($p < $l):
							$avp = $p;
						endif;
						/* Поиск конца значения атрибута. */
						$p = iaSkipTo ($s, $q, $p, $l);
						if ($p !== FALSE):
							$avl = $p - $avp;
							$p++;	// пропуск закрывающей кавычки
						endif;
					else:
						$avp = $p;
						$p = iaSkipTo ($s, ">$spc", $p, $l);
						if ($p !== FALSE):
							$avl = $p - $avp;
						endif;
						/* Значение атрибута записано без кавычек. */
					endif;
					/* Пропуск пробелов после конца значения атрибута. */
					$p = iaSkipOver ($s, $spc, $p, $l);
				endif;
			endif;
			/* Сохранение информации о найденном атрибуте. */
			if ($tag['attr'] === FALSE):
				$tag['attr'] = array ();
			endif;
			$an = $anl > 0? substr ($s, $anp, $anl): FALSE;
			$av = ($avp > -1 and $avl) > 0? substr ($s, $avp, $avl): FALSE;
			$tag['attr'][$an] = $av;
		endwhile;
		$tag['len'] = ($p !== FALSE? $p + 1: $l) - $tag['pos'];
	endif;
	return $tag;
}
?>